
/*
 * DEFS.H
 *
 *
 *
 * (c)Copyright 1997, Matthew Dillon, All Rights Reserved.  Refer to
 *    the COPYRIGHT file in the base directory of this distribution
 *    for specific rights granted.
 */

#include "config.h"

#include <sys/types.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <sys/socket.h>		/* internet sockets	*/
#include <sys/un.h>		/* unix domain sockets	*/
#include <sys/mman.h>		/* mmap()		*/
#include <netinet/in.h>		/* internet sockets	*/
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#if USE_BSTRING_H
#include <bstring.h>
#endif

#include <stdarg.h>
#include <stddef.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <netdb.h>		/* internet sockets	*/
#include <arpa/inet.h>
#include <signal.h>
#include <syslog.h>
#include <fcntl.h>
#include <pwd.h>
#include <grp.h>
#include <math.h>

#define Prototype extern

typedef struct {
    long	h1;
    long	h2;
} hash_t;

typedef struct History {
    unsigned long	next;	/* next link		*/
    unsigned long	gmt;	/* gmt time in minutes	*/
    hash_t		hv;	/* hash value		*/
    unsigned short	iter;	/* iteration		*/
    unsigned short	exp;	/* hours relative to gmt minutes */
} History;

#ifdef FUTURE

typedef struct Active {
    unsigned long	next;	/* next link		*/
    unsigned long	sub;	/* subhierarchy		*/
    unsigned long	loWat;	/* low water mark	*/
    unsigned long	hiWat;	/* high water mark	*/
    unsigned long	id;	/* Overview file id	*/
    unsigned short	bytes;
    char		data[2];
} Active;

typedef struct OverRef {
    unsigned long	offset;	/* offset in file -> OverView		*/
    unsigned short	artMod;	/* article number MSB mod value		*/
    unsigned short	reserved;
} OverRef;

typedef struct OverHead {
    int			arts;	/* (fixed) max articles for this group	*/
    int			rollov;	/* roll-over point			*/
    OverRef		ref[1];	/* index array			        */
} OverHead;

typedef struct OverView {
    int			artno;		/* article number double-check	*/
    short		nelm;		/* number of elements		*/
    short		reserved;	/* future			*/
    short		elen[1];	/* element byte lens [nelm]	*/
    /* elen[N]		*/
    /* elements		*/
} OverView;

#endif

#define HGF_FAST	0x01
#define HGF_NOSEARCH	0x02

typedef struct Node {
    struct Node		*no_Next;
    int			no_Value;
    const char		*no_Name;
    const char		*no_Name2;
} Node;

#define MAXLINE		16384
#define MAXMSGIDLEN	512
#define MAXGNAME	256
#define MAXFORKS	256
#define MAXFEEDS	128
#define MAXFDS		(MAXFORKS+MAXFEEDS+32)

#ifndef HSIZE

#if USE_SMALL_HASH
#define HSIZE		(1024*1024)		
#else
#if USE_LARGE_HASH
#define HSIZE		(4096*1024)
#else
#define HSIZE		(1024*1024)	/* will change to 4million >= V1.10 */
#endif
#endif

#endif

#define MAXSIMUL	5	/* maximum simultanious from same label */

#define VERSION		"1.00"

#define RCOK		0	/* article accepted		*/
#define RCALREADY	1	/* we already have this article	*/
#define RCTRYAGAIN	2	/* something went wrong, requeue it for later */
#define RCREJECT	3	/* there is something wrong with your article */
#define RCERROR		4	/* an unexpected error occured  */

#define XLOCK_SH	1
#define XLOCK_EX	2
#define XLOCK_UN	3
#define XLOCK_NB	0x10

#define MAXFEEDTABLE	256

typedef struct Buffer {
    int			bu_Beg;
    int			bu_NLScan;
    int			bu_End;
    int			bu_Max;
    char		bu_Data[1024];
} Buffer;

/*
 * MISC
 */

#ifndef LOG_PERROR
#define LOG_PERROR	0
#endif


#include "protos.h"

