
/*
 * LIB/NODE.C	- Simple string node routines
 *
 *
 * (c)Copyright 1997, Matthew Dillon, All Rights Reserved.  Refer to
 *    the COPYRIGHT file in the base directory of this distribution 
 *    for specific rights granted.
 *
 */

#include "defs.h"

Prototype void FreeNodeList(Node **pnode);
Prototype void FreeNode(Node *node);
Prototype Node *MakeNode(const char *str, int val);
Prototype Node *MakeNode2(const char *str1, const char *str2, int val);

const char *GetStrTab(const char *s);
void RelStrTab(const char *s);
int strhash(const char *s);

void
FreeNodeList(Node **pnode)
{
    Node *no;

    while ((no = *pnode) != NULL) {
	*pnode = no->no_Next;
	FreeNode(no);
    }
}

void
FreeNode(Node *node)
{
    if (node->no_Name) {
        RelStrTab(node->no_Name);
        node->no_Name = NULL;
    }
    if (node->no_Name2) {
        RelStrTab(node->no_Name2);
        node->no_Name2 = NULL;
    }
    free(node);
}

Node *
MakeNode(const char *str, int val)
{
    Node *node = malloc(sizeof(Node));

    node->no_Next = NULL;
    node->no_Name = GetStrTab(str);
    node->no_Name2 = NULL;
    node->no_Value = val;
    return(node);
}

Node *
MakeNode2(const char *str1, const char *str2, int val)
{
    Node *node = malloc(sizeof(Node));

    node->no_Next = NULL;
    node->no_Name = GetStrTab(str1);
    node->no_Name2 = GetStrTab(str2);
    node->no_Value = val;

    return(node);
}

/*
 *
 */

#define STHSIZE         128
#define STHMASK         (STHSIZE-1)

typedef struct STab {
    struct STab *st_Next;
    int         st_Refs;
    int         st_Hv;
} STab;

STab    *STHash[STHSIZE];

const char *
GetStrTab(const char *s)
{
    int hv = strhash(s);
    STab **pst = &STHash[hv & STHMASK];
    STab *st;

    while ((st = *pst) != NULL) {
        if (st->st_Hv == hv && strcmp((char *)(st + 1), s) == 0)
            break;
        pst = &st->st_Next;
    }
    if (st == NULL) {
        st = *pst = malloc(sizeof(STab) + strlen(s) + 1);
        st->st_Next = NULL;
        st->st_Hv = hv;
        st->st_Refs = 0;
        strcpy((char *)(st + 1), s);
    }
    ++st->st_Refs;
    return((char *)(st + 1));
}

void
RelStrTab(const char *s)
{
    int hv = strhash(s);
    STab **pst = &STHash[hv & STHMASK];
    STab *st;

    while ((st = *pst) != NULL) {
        if (st->st_Hv == hv && strcmp((char *)(st + 1), s) == 0)
            break;
        pst = &st->st_Next;
    }
    if (st && --st->st_Refs == 0) {
        *pst = st->st_Next;
        free(st);
    }
}

int
strhash(const char *s)
{
    int hv = 0xA45CD32F;

    while (*s) {
        hv = (hv << 5) ^ *s ^ (hv >> 23);
        ++s;
    }
    return(hv ^ (hv > 16));
}

