
/*
 * LIB/XOPEN.C
 *
 * (c)Copyright 1997, Matthew Dillon, All Rights Reserved.  Refer to
 *    the COPYRIGHT file in the base directory of this distribution 
 *    for specific rights granted.
 *
 */

#include "defs.h"

Prototype int xopen(int modes, int perms, const char *ctl, ...);
Prototype FILE *xfopen(const char *modes, const char *ctl, ...);
Prototype int xfprintf(FILE *fo, const char *ctl, ...);
Prototype void dprintf(const char *ctl, ...);
Prototype void vdprintf(const char *ctl, va_list va);

int
xopen(int modes, int perms, const char *ctl, ...)
{
    char path[1024];
    va_list va;

    va_start(va, ctl);
#if HAVE_SNPRINTF == 0
    vsprintf(path, ctl, va);
#else
    vsnprintf(path, sizeof(path), ctl, va);
#endif
    va_end(va);
    return(open(path, modes, perms));
}


FILE *
xfopen(const char *modes, const char *ctl, ...)
{
    char path[1024];
    va_list va;

    va_start(va, ctl);
#if HAVE_SNPRINTF == 0
    vsprintf(path, ctl, va);
#else
    vsnprintf(path, sizeof(path), ctl, va);
#endif
    va_end(va);
    return(fopen(path, modes));
}

int
xfprintf(FILE *fo, const char *ctl, ...)
{
    va_list va;
    int r;

    if (fo == NULL) {
	syslog(LOG_CRIT, "xfprintf() - fo NULL!");
	exit(1);
    }

    va_start(va, ctl);
    r = vfprintf(fo, ctl, va);
    va_end(va);
    if (DebugOpt) {
	dprintf("%d >> ", (int)getpid());
	va_start(va, ctl);
	vdprintf(ctl, va);
	va_end(va);
    }
    return(r);
}

void 
dprintf(const char *ctl, ...)
{
    va_list va;

    if (DebugOpt) {
	va_start(va, ctl);
	vdprintf(ctl, va);
	va_end(va);
    }
}

void 
vdprintf(const char *ctl, va_list va)
{
    if (DebugOpt) {
	vprintf(ctl, va);
    }
}

