
/*
 * DIDUMP.C	Dump or trace a dhistory file
 *
 * DIDUMP [-t] [-x] dhistory
 *
 * (c)Copyright 1997, Matthew Dillon, All Rights Reserved.  Refer to
 *    the COPYRIGHT file in the base directory of this distribution 
 *    for specific rights granted.
 */

#include "defs.h"

int TraceMode;
int DNPOpt;
int FOpt;

void DumpTrace(int fd, int hsize);
void DumpQuick(int fd, int hsize);

int
main(int ac, char **av)
{
    int fd;
    int i;
    int hsize = 1024 * 1024;
    struct stat st;
    char *fileName = NULL;
    char *paramName = NULL;

    for (i = 1; i < ac; ++i) {
	char *ptr = av[i];

	if (*ptr != '-') {
	    if (fileName) {
		fprintf(stderr, "unexpected argument\n");
		exit(1);
	    }
	    fileName = ptr;
	    continue;
	}
	ptr += 2;
	switch(ptr[-1]) {
	case 'f':
	    FOpt = 1;
	    break;
	case 'x':
	    DNPOpt = 1;
	    break;
	case 't':
	    TraceMode = 1;
	    break;
	default:
	    fprintf(stderr, "illegal option: %s\n", ptr - 2);
	    exit(1);
	}
    }

    if (fileName == NULL) {
	printf("didump [-t] [-x] dhistory-file\n");
	printf("    default	- quick dump\n");
	printf("    -t		- hash table trace (slow)\n");
	printf("    -x		- dump / DNP entries over 16 days old\n");
	exit(0);
    }

    paramName = malloc(strlen(fileName) + 32);
    sprintf(paramName, "%s.param", fileName);

    {
	FILE *fi;

	if ((fi = fopen(paramName, "r")) != NULL) {
	    char buf[256];

	    hsize = 0;

	    while (fgets(buf, sizeof(buf), fi) != NULL) {
		if (buf[0] == 'H')
		    hsize = strtol(buf + 1, NULL, 0);
	    }
	    fclose(fi);

	    if (hsize == 0) {
		fprintf(stderr, "dhistory parameter file error\n");
		exit(1);
	    }
	}
    }

    if (DNPOpt && TraceMode) {
	fprintf(stderr, "-x only works for quick dumps\n");
	exit(1);
    }

    if ((fd = open(fileName, O_RDONLY)) >= 0 && fstat(fd, &st) == 0) {
	if (TraceMode)
	    DumpTrace(fd, hsize);
	else
	    DumpQuick(fd, hsize);
	close(fd);
    } else {
	fprintf(stderr, "open failed\n");
	exit(1);
    }
    return(0);
}

void
DumpTrace(int fd, int hsize)
{
    int i;
    long *Ary = calloc(hsize, sizeof(long));

    if (read(fd, Ary, hsize * sizeof(long)) != hsize * sizeof(long)) {
	printf("Unable to read hash table array\n");
	exit(1);
    }

    for (i = 0; i < hsize; ++i) {
	long off;

	if ((off = Ary[i]) != 0) {
	    long loff = hsize * sizeof(long) - 4;
	    int maxChainLen = 1000;

	    printf("Index %d: ", i);

	    while (off) {
		History h;

		lseek(fd, off, 0);
		if (read(fd, &h, sizeof(h)) != sizeof(h)) {
		    printf("read error @ %ld", off);
		    break;
		} else {
		    printf(" [%ld,%ld %08lx.%08lx.%04x gm=%d ex=%d]",
			off, h.next,
			h.hv.h1,
			h.hv.h2,
			(int)h.iter,
			(int)h.gmt,
			(int)h.exp
		    );
		}
		loff = off;
		off = h.next;
		if (--maxChainLen == 0) {
		    printf(" MAXIMEM CHAIN LENGTH EXCEEDED!");
		    break;
		}
	    }
	    if (off)
		printf(" offset error: %ld", off);
	    printf("\n");
	}
    }
}

void
DumpQuick(int fd, int hsize)
{
    int n;
    History HAry[4096];
    unsigned long gmt = time(NULL) / 60;

    if (FOpt)
	lseek(fd, 0L, 2);
    else
	lseek(fd, hsize * sizeof(long), 0);

top:
    while ((n = read(fd, HAry, sizeof(HAry))) > 0) {
	int i;

	n /= sizeof(HAry[0]);
	for (i = 0; i < n; ++i) {
	    History *h = &HAry[i];

	    if ((i & 255) == 0)
		gmt = time(NULL) / 60;

	    if (DNPOpt && h->exp == (unsigned short)-1) {
		long dgmt = gmt - h->gmt;	/* DELTA MINUTES */
		if (dgmt < -16 * 24 * 60 || dgmt > 16 * 24 * 60)
		    continue;
	    }

	    printf("DUMP %08lx.%08lx.%04x gm=%d ex=%d\n",
		h->hv.h1,
		h->hv.h2,
		(int)h->iter,
		(int)h->gmt,
		(int)h->exp
	    );
	}
    }
    if (FOpt) {
	sleep(1);
	goto top;
    }
    printf(".\n");
}

