/*
 * DSPOOLOUT.C
 *
 * Flush the outbound queue files and start up dnewslinks as appropriate
 *
 * (c)Copyright 1997, Matthew Dillon, All Rights Reserved.  Refer to
 *    the COPYRIGHT file in the base directory of this distribution 
 *    for specific rights granted.
 */

#include "defs.h"

#define DIABLO

#ifndef OUTGOING
#ifdef DIABLO
#define OUTGOING	"/news/dqueue"
#else
#define OUTGOING	"/news/spool/out.going"
#endif
#endif

#ifndef NNTPSPOOLCTL
#ifdef DIABLO
#define NNTPSPOOLCTL	"dnntpspool.ctl"
#else
#define NNTPSPOOLCTL	"nntpspool.ctl"
#endif
#endif

#ifndef NEWSHOME
#define NEWSHOME	"/news"
#endif

#define TYPE_NORMAL	1
#define TYPE_XREPLIC	2

int MaxRun = 2;		/* default MaxRun				       */
int MinFlushSecs = 0;	/* minimum time between flushes if queue not caught up */

typedef struct DNode {
    struct DNode *no_Next;
    char	*no_SpoolFile;
} DNode;

DNode *Base;

void spool_file(char *spoolFile, char *hostName, int maxq, int maxr, int type, int pass);
DNode *FindNode(char *name);
void AddNode(char *name);

int Verbose;
int Quiet;

int
main(int ac, char **av)
{
    char buf[256];
    FILE *fi;
    int pass;

    /*
     * Shift into the out.going directory
     */

    if (chdir(OUTGOING) != 0) {
	fprintf(stderr, "unable to chdir to %s\n", OUTGOING);
	exit(1);
    }

    {
	int i;

	for (i = 1; i < ac; ++i) {
	    char *ptr = av[i];
	    if (*ptr == '-') {
		ptr += 2;
		switch(ptr[-1]) {
		case 'q':
		    Quiet = 1;
		    break;
		case 'v':
		    Verbose = 1;
		    break;
		case 's':
		    MinFlushSecs = strtol((*ptr ? ptr : av[++i]), NULL, 0) * 60;
		    break;
		case 'm':
		    MaxRun = strtol((*ptr ? ptr : av[++i]), NULL, 0);
		    break;
		default:
		    break;
		}
	    }
	}
    }

    /*
     * Get list of spoolfile names and NNTP hosts, one pair per line,
     * blank lines and lines beginning with '#' excepted.
     */

    for (pass = 1; pass <= 2; ++pass) {
	if ((fi = fopen(NEWSHOME "/" NNTPSPOOLCTL, "r")) != NULL) {
	    while (fgets(buf, sizeof(buf), fi) != NULL) {
		char *spoolFile;
		char *hostName;

		if (buf[0] == '\n' || buf[0] == '#')
		    continue;
		if ((spoolFile = strtok(buf, " \t\n")) != NULL &&
		    (hostName = strtok(NULL, " \t\n")) != NULL
		) {
		    int maxq = -1;
		    int maxr = MaxRun;
		    int type = TYPE_NORMAL;

		    {
			char *maxqStr;
			while ((maxqStr = strtok(NULL, " \t\n")) != NULL) {
			    if (strtol(maxqStr, NULL, 0) > 0) {
				maxq = strtol(maxqStr, NULL, 0);
			    } else if (strcmp(maxqStr, "xreplic") == 0) {
				type = TYPE_XREPLIC;
			    } else if (maxqStr[0] == 'n') {
				maxr = strtol(maxqStr + 1, NULL, 0);
			    } else {
				if (pass == 1)
				    fprintf(stderr, "bad keyword in control file: %s\n", maxqStr);
			    }
			}
		    }
		    spool_file(spoolFile, hostName, maxq, maxr, type, pass);
		}
	    }
	    fclose(fi);
	} else if (pass == 1) {
	    fprintf(stderr, "Unable to open " NEWSHOME "/nntpspool.ctl\n");
	}
	if (pass == 1) {
	    char sysline[256];

	    sprintf(sysline, "/news/dbin/dicmd flush");
	    system(sysline);
	}
    }
    return(0);
}

/*
 * SPOOL_FILE() - pass 1 - rename spool files
 *
 *		  pass 2 - start dnewslink processes
 */

void
spool_file(char *spoolFile, char *hostName, int maxq, int maxr, int type, int pass)
{
    char seqFile[256];
    char newFile[256];
    int begSeq = 0;
    int endSeq = 0;
    long newTime = 0;
    int fd;
    struct stat st;

    sprintf(seqFile, ".%s.seq", spoolFile);

    /*
     * Get beginning and ending sequence numbers
     */

    fd = open(seqFile, O_RDWR|O_CREAT, 0600);

    if (fd >= 0) {
	char buf[64];

	xflock(fd, XLOCK_EX);		/* lock and leave locked */
	memset(buf, 0, sizeof(buf));
	read(fd, buf, sizeof(buf) - 1);
	sscanf(buf, "%d %d %lx", &begSeq, &endSeq, &newTime);
    }

    /*
     * Discard beginning sequence numbers that are now deleted, or
     * delete queue files that are too old.
     */

    while (begSeq < endSeq) {
	sprintf(newFile, "%s.S%05d", spoolFile, begSeq);
	if (maxq > 0 && begSeq < endSeq - maxq)
	    remove(newFile);
	if (stat(newFile, &st) == 0)
	    break;
	++begSeq;
    }

    /*
     * If primary spool file exists, shift to primary
     * queue
     */

    if (pass == 1) {
	{
	    int tries = 100;
	    long dt;

	    while (tries > 0) {
		struct stat st;

		sprintf(newFile, "%s.S%05d", spoolFile, endSeq);
		if (stat(newFile, &st) < 0)
		    break;
		--tries;
		++endSeq;
	    }

	    /*
	     * Only create a new spool file if:
	     *
	     * (1) endSeq == begSeq + 1, or
	     * (2) current time larger then newTime + MinFlushSecs
	     * (3) there is time weirdness
	     */

	    dt = time(NULL) - (newTime + MinFlushSecs);

	    if (endSeq <= begSeq + 2 || dt > 0 || dt < -MinFlushSecs) {
		if (rename(spoolFile, newFile) == 0) {
		    ++endSeq;
		    if (!Quiet)
			printf("dspoolout: add %s\n", newFile);
		} else {
		    if (Verbose && !Quiet)
			printf("dspoolout: nofile %s\n", newFile);
		    if (endSeq == begSeq)
			AddNode(spoolFile);
		}
		newTime = time(NULL);
	    } else {
		AddNode(spoolFile);
		if (Verbose && !Quiet)
		    printf("dspoolout: wait %s\n", spoolFile);
	    }
	}

	/*
	 * Update sequence number file
	 */

	if (fd >= 0) {
	    char buf[64];

	    sprintf(buf, "%5d %5d %08lx\n", begSeq, endSeq, newTime);
	    lseek(fd, 0L, 0);
	    write(fd, buf, strlen(buf));
	    ftruncate(fd, lseek(fd, 0L, 1));
	}
    }

    if (fd >= 0) {
	xflock(fd, XLOCK_UN);
	close(fd);
    }

    /*
     * Run up to N newslink's/innxmit's, usually set at 2.  More may actually
     * be running: the nntplink from the INN channel, and older nntplink's
     * from previously flushed INN channels that have not reached EOF
     * on their pipe.
     *
     * Note that we spool out the queue as a FIFO.  This is important as
     * it reduces article jumble.
     */

    if (pass == 2 && FindNode(spoolFile) == NULL) {
	int tries = maxr;
	int look;
	int fd;

	for (look = 0; look < 10 && tries && begSeq + look < endSeq; ++look) {
	    int use = begSeq + look;

	    sprintf(newFile, "%s.S%05d", spoolFile, use);
	    if ((fd = open(newFile, O_RDWR)) >= 0) {
		if (xflock(fd, XLOCK_EX|XLOCK_NB) == 0) {
		    char seqName[64];
		    char templateFile[256];

		    xflock(fd, XLOCK_UN);

		    if (!Quiet)
			printf("dspoolout: run %s.S%05d\n", spoolFile, use);

		    if (fork() == 0) {
			sprintf(templateFile, "%s.S%%05d", spoolFile);
			sprintf(seqName, "%d", use);

			switch(type) {
			case TYPE_NORMAL:
#ifdef DIABLO
			    execlp("/news/dbin/dnewslink", "dnewslink", 
#else
			    execlp("newslink", "newslink", 
#endif
				"-h", hostName,
				"-b", templateFile,
				"-S", seqName,
				"-N", "32",
				"-D",
				NULL
			    );
			    break;
			case TYPE_XREPLIC:
			    execlp("innxmit", "innxmit",
				"-a",
				"-r",
				"-S", hostName,
				newFile,
				NULL
			    );
			    break;
			}
			exit(0);
		    }
		}
		--tries;	/* even if lock fails */
		close(fd);
	    }
	}
    }
}

DNode *
FindNode(char *name)
{
    DNode *node;

    for (node = Base; node; node = node->no_Next) {
	if (strcmp(name, node->no_SpoolFile) == 0)
	    break;
    }
    return(node);
}

void
AddNode(char *name)
{
    DNode *node = malloc(sizeof(Node) + strlen(name) + 1);

    node->no_Next = Base;
    node->no_SpoolFile = (char *)(node + 1);
    Base = node;
    strcpy(node->no_SpoolFile, name);
}

